@@ -30,13 +30,11 @@ from .sqlclitransactionmanager import TransactionManager
 from .sqlexecute import SQLExecute
 from .sqlparse import SQLMapping
 from .kafkawrapper import KafkaWrapper
-from .comparewrapper import CompareWrapper
+from .testwrapper import TestWrapper
 from .hdfswrapper import HDFSWrapper
 from .sqlcliexception import SQLCliException
 from .sqlclisga import SQLCliGlobalSharedMemory
-from .sqlinternal import Create_file
-from .sqlinternal import Convert_file
-from .sqlinternal import Create_SeedCacheFile
+from .datawrapper import DataWrapper
 from .commandanalyze import register_special_command
 from .commandanalyze import CommandNotFound
 from .sqloption import SQLOptions
@@ -95,19 +93,18 @@ class SQLCli(object):
             resultcharset='UTF-8',
             SharedProcessInfo=None,
             profile=None,
-            reffile=None,
-            rptfile=None
     ):
         self.db_saved_conn = {}                          # 数据库Session备份
         self.SQLMappingHandler = SQLMapping()            # 函数句柄，处理SQLMapping信息
         self.SQLExecuteHandler = SQLExecute()            # 函数句柄，具体来执行SQL
         self.SQLOptions = SQLOptions()                   # 程序运行中各种参数
         self.KafkaHandler = KafkaWrapper()               # Kafka消息管理器
-        self.CompareHandler = CompareWrapper()           # 参照结果文件比对
+        self.TestHandler = TestWrapper()                 # 测试管理
         self.HdfsHandler = HDFSWrapper()                 # HDFS文件操作
         self.JobHandler = JOBManager()                   # 并发任务管理器
         self.TransactionHandler = TransactionManager()   # 事务管理器
-        self.SpoolFileHandler = None                     # 当前Spool文件句柄
+        self.DataHandler = DataWrapper()                 # 随机临时数处理
+        self.SpoolFileHandler = []                       # Spool文件句柄, 是一个数组，可能发生嵌套
         self.EchoFileHandler = None                      # 当前回显文件句柄
         self.AppOptions = None                           # 应用程序的配置参数
         self.Encoding = None                             # 应用程序的Encoding信息
@@ -125,11 +122,9 @@ class SQLCli(object):
             self.Result_Charset = self.Client_Charset    # 结果输出字符集
         else:
             self.Result_Charset = resultcharset
-        self.WorkerName = WorkerName                        # 当前进程名称. 如果有参数传递，以参数为准
-        self.MultiProcessManager = None                     # 进程间共享消息管理器， 如果为子进程，该参数为空
-        self.profile = None                                 # 程序的初始化日志文件
-        self.reffile = None                                 # 程序的结果参照文件
-        self.rptfile = None                                 # 程序的结果报告文件
+        self.WorkerName = WorkerName                     # 当前进程名称. 如果有参数传递，以参数为准
+        self.MultiProcessManager = None                  # 进程间共享消息管理器， 如果为子进程，该参数为空
+        self.profile = []                                # 程序的初始化日志文件
 
         # 传递各种参数
         self.sqlscript = sqlscript
@@ -154,25 +149,23 @@ class SQLCli(object):
             self.SharedProcessInfo = obj()
         else:
             self.SharedProcessInfo = SharedProcessInfo
-        if profile is None:
-            # profile的顺序， SQLCLI_HOME/profile/default,   <PYTHON_PACKAGE>/sqlcli/profile/default, user define
-            if "SQLCLI_HOME" in os.environ:
-                if os.path.exists(os.path.join(os.environ["SQLCLI_HOME"], "profile", "default")):
-                    self.profile = os.path.join(os.environ["SQLCLI_HOME"], "profile", "default")
-            if self.profile is None:
-                if os.path.exists(os.path.join(os.path.dirname(__file__), "profile", "default")):
-                    self.profile = os.path.join(os.path.dirname(__file__), "profile", "default")
-        else:
-            if os.path.exists(profile):
-                self.profile = profile
+
+        # profile的顺序， <PYTHON_PACKAGE>/sqlcli/profile/default， SQLCLI_HOME/profile/default , user define
+        if os.path.isfile(os.path.join(os.path.dirname(__file__), "profile", "default")):
+            if os.path.getsize(os.path.join(os.path.dirname(__file__), "profile", "default")) > 0:
+                self.profile.append(os.path.join(os.path.dirname(__file__), "profile", "default"))
+        if "SQLCLI_HOME" in os.environ:
+            if os.path.isfile(os.path.join(os.environ["SQLCLI_HOME"], "profile", "default")):
+                self.profile.append(os.path.join(os.environ["SQLCLI_HOME"], "profile", "default"))
+        if profile is not None:
+            if os.path.isfile(profile):
+                self.profile.append(profile)
             else:
                 if "SQLCLI_DEBUG" in os.environ:
                     print("Profile does not exist ! Will ignore it. [" + str(os.path.abspath(profile)) + "]")
         if "SQLCLI_DEBUG" in os.environ:
-            if self.profile is not None:
-                print("Profile = [" + str(self.profile) + "]")
-        self.reffile = reffile
-        self.rptfile = rptfile
+            for m_Profile in self.profile:
+                print("Profile = [" + str(m_Profile) + "]")
 
         # 设置self.JobHandler， 默认情况下，子进程启动的进程进程信息来自于父进程
         self.JobHandler.setProcessContextInfo("logon", self.logon)
@@ -221,7 +214,7 @@ class SQLCli(object):
         register_special_command(
             self.load_driver,
             command="loaddriver",
-            description="load SQL Driver file.",
+            description="加载数据库驱动文件",
             hidden=False
         )
 
@@ -229,7 +222,7 @@ class SQLCli(object):
         register_special_command(
             self.load_sqlmap,
             command="loadsqlmap",
-            description="load SQL Mapping file .",
+            description="加载SQL映射文件",
             hidden=False
         )
 
@@ -237,7 +230,7 @@ class SQLCli(object):
         register_special_command(
             handler=self.connect_db,
             command="connect",
-            description="Connect to database .",
+            description="连接到指定的数据库",
             hidden=False
         )
 
@@ -245,7 +238,7 @@ class SQLCli(object):
         register_special_command(
             handler=self.session_manage,
             command="session",
-            description="Manage connect sessions.",
+            description="数据库连接会话管理",
             hidden=False
         )
 
@@ -253,7 +246,7 @@ class SQLCli(object):
         register_special_command(
             handler=self.disconnect_db,
             command="disconnect",
-            description="Disconnect database .",
+            description="断开数据库连接",
             hidden=False
         )
 
@@ -261,7 +254,7 @@ class SQLCli(object):
         register_special_command(
             self.execute_from_file,
             command="start",
-            description="Execute commands from file.",
+            description="执行指定的测试脚本",
             hidden=False
         )
 
@@ -269,7 +262,7 @@ class SQLCli(object):
         register_special_command(
             self.sleep,
             command="sleep",
-            description="Sleep some time (seconds)",
+            description="程序休眠(单位是秒)",
             hidden=False
         )
 
@@ -277,7 +270,7 @@ class SQLCli(object):
         register_special_command(
             self.set_options,
             command="set",
-            description="set options .",
+            description="设置运行时选项",
             hidden=False
         )
 
@@ -285,7 +278,7 @@ class SQLCli(object):
         register_special_command(
             self.spool,
             command="spool",
-            description="spool output to a file",
+            description="将输出打印到指定文件",
             hidden=False
         )
 
@@ -293,7 +286,7 @@ class SQLCli(object):
         register_special_command(
             self.echo_input,
             command="echo",
-            description="ECHO input into file",
+            description="回显输入到指定的文件",
             hidden=False
         )
 
@@ -301,7 +294,7 @@ class SQLCli(object):
         register_special_command(
             self.host,
             command="host",
-            description="execute OS command.",
+            description="执行操作系统命令",
             hidden=False
         )
 
@@ -309,7 +302,13 @@ class SQLCli(object):
         register_special_command(
             self.execute_internal_command,
             command="__internal__",
-            description="execute internal command.",
+            description="执行内部操作命令：\n"
+                        "    __internal__ hdfs          HDFS文件操作\n"
+                        "    __internal__ kafka         kafka队列操作\n"
+                        "    __internal__ data          随机测试数据管理\n"
+                        "    __internal__ test          测试管理\n"
+                        "    __internal__ job           后台并发测试任务管理\n"
+                        "    __internal__ transaction   后台并发测试事务管理",
             hidden=False
         )
 
@@ -317,7 +316,7 @@ class SQLCli(object):
         register_special_command(
             self.exit,
             command="exit",
-            description="Exit program.",
+            description="正常退出当前应用程序",
             hidden=False
         )
 
@@ -896,12 +895,12 @@ class SQLCli(object):
         parameters = str(arg).split()
         if parameters[0].strip().upper() == 'OFF':
             # close spool file
-            if self.SpoolFileHandler is None:
+            if len(self.SpoolFileHandler) == 0:
                 message = "not spooling currently"
                 return [(None, None, None, None, message)]
             else:
-                self.SpoolFileHandler.close()
-                self.SpoolFileHandler = None
+                self.SpoolFileHandler[-1].close()
+                self.SpoolFileHandler.pop()
                 self.SQLExecuteHandler.spoolfile = None
                 return [(None, None, None, None, None)]
 
@@ -911,11 +910,10 @@ class SQLCli(object):
         else:
             # 如果主程序没有启用日志，则输出为当前目录
             m_FileName = parameters[0].strip()
+
         # 如果当前有打开的Spool文件，关闭它
-        if self.SpoolFileHandler is not None:
-            self.SpoolFileHandler.close()
         try:
-            self.SpoolFileHandler = open(m_FileName, "w", encoding=self.Result_Charset)
+            self.SpoolFileHandler.append(open(m_FileName, "w", encoding=self.Result_Charset))
         except IOError as e:
             raise SQLCliException("SQLCLI-00000: IO Exception " + repr(e))
         self.SQLExecuteHandler.spoolfile = self.SpoolFileHandler
@@ -941,7 +939,7 @@ class SQLCli(object):
         # ECHO的输出默认为程序的工作目录
         m_FileName = parameters[0].strip()
 
-        # 如果当前有打开的Spool文件，关闭它
+        # 如果当前有打开的Echo文件，关闭它
         if self.EchoFileHandler is not None:
             self.EchoFileHandler.close()
         self.EchoFileHandler = open(m_FileName, "w", encoding=self.Result_Charset)
@@ -955,7 +953,8 @@ class SQLCli(object):
         elif arg == "":      # 显示所有的配置
             m_Result = []
             for row in self.SQLOptions.getOptionList():
-                m_Result.append([row["Name"], row["Value"], row["Comments"]])
+                if not row["Hidden"]:
+                    m_Result.append([row["Name"], row["Value"], row["Comments"]])
             yield (
                 "Current Options: ",
                 m_Result,
@@ -1059,10 +1058,10 @@ class SQLCli(object):
             yield title, result, headers, columntypes, status
             return
 
-        # 运行日志比对
-        matchObj = re.match(r"(\s+)?compare(.*)$", arg, re.IGNORECASE | re.DOTALL)
+        # 测试管理
+        matchObj = re.match(r"(\s+)?test(.*)$", arg, re.IGNORECASE | re.DOTALL)
         if matchObj:
-            (title, result, headers, columntypes, status) = self.CompareHandler.Process_SQLCommand(arg)
+            (title, result, headers, columntypes, status) = self.TestHandler.Process_SQLCommand(arg)
             yield title, result, headers, columntypes, status
             return
 
@@ -1073,102 +1072,12 @@ class SQLCli(object):
             yield title, result, headers, columntypes, status
             return
 
-        # 创建数据文件, 根据末尾的rows来决定创建的行数
-        # 此时，SQL语句中的回车换行符没有意义
-        matchObj = re.match(r"create\s+(.*?)\s+file\s+(.*?)\((.*)\)(\s+)?rows\s+(\d+)(\s+)?$",
-                            arg, re.IGNORECASE | re.DOTALL)
+        # 处理随机数据文件
+        matchObj = re.match(r"(\s+)?data(.*)$", arg, re.IGNORECASE | re.DOTALL)
         if matchObj:
-            m_filetype = str(matchObj.group(1)).strip()
-            m_filename = str(matchObj.group(2)).strip().replace('\r', '').replace('\n', '')
-            m_formula_str = str(matchObj.group(3).replace('\r', '').replace('\n', '').strip())
-            m_rows = int(matchObj.group(5))
-            Create_file(p_filetype=m_filetype,
-                        p_filename=m_filename,
-                        p_formula_str=m_formula_str,
-                        p_rows=m_rows,
-                        p_encoding=self.Result_Charset)
-            yield (
-                None,
-                None,
-                None,
-                None,
-                str(m_rows) + ' rows created Successful.')
-            return
-
-        matchObj = re.match(r"create\s+(.*?)\s+file\s+(.*?)\((.*)\)(\s+)?$",
-                            arg, re.IGNORECASE | re.DOTALL)
-        if matchObj:
-            m_filetype = str(matchObj.group(1)).strip()
-            m_filename = str(matchObj.group(2)).strip().replace('\r', '').replace('\n', '')
-            m_formula_str = str(matchObj.group(3).strip())
-            m_rows = 1
-            Create_file(p_filetype=m_filetype,
-                        p_filename=m_filename,
-                        p_formula_str=m_formula_str,
-                        p_rows=m_rows,
-                        p_encoding=self.Result_Charset)
-            yield (
-                None,
-                None,
-                None,
-                None,
-                str(m_rows) + ' rows created Successful.')
-            return
-
-        #  在不同的文件中进行相互转换
-        matchObj = re.match(r"create\s+(.*?)\s+file\s+(.*?)\s+from\s+(.*?)file(.*?)(\s+)?$",
-                            arg, re.IGNORECASE | re.DOTALL)
-        if matchObj:
-            # 在不同的文件中相互转换
-            Convert_file(p_srcfileType=str(matchObj.group(3)).strip(),
-                         p_srcfilename=str(matchObj.group(4)).strip(),
-                         p_dstfileType=str(matchObj.group(1)).strip(),
-                         p_dstfilename=str(matchObj.group(2)).strip())
-            yield (
-                None,
-                None,
-                None,
-                None,
-                'file converted Successful.')
-            return
-
-        # 创建随机数Seed的缓存文件
-        matchObj = re.match(r"create\s+(integer|string)\s+seeddatafile\s+(.*?)\s+"
-                            r"length\s+(\d+)\s+rows\s+(\d+)\s+with\s+null\s+rows\s+(\d+)$",
-                            arg, re.IGNORECASE | re.DOTALL)
-        if matchObj:
-            m_DataType = str(matchObj.group(1)).lstrip().rstrip()
-            m_SeedFileName = str(matchObj.group(2)).lstrip().rstrip()
-            m_DataLength = int(matchObj.group(3))
-            m_nRows = int(matchObj.group(4))
-            m_nNullValueCount = int(matchObj.group(5))
-            Create_SeedCacheFile(p_szDataType=m_DataType, p_nDataLength=m_DataLength, p_nRows=m_nRows,
-                                 p_szSeedName=m_SeedFileName, p_nNullValueCount=m_nNullValueCount)
-            yield (
-                None,
-                None,
-                None,
-                None,
-                'seed file created Successful.')
-            return
-
-        # 创建随机数Seed的缓存文件
-        matchObj = re.match(r"create\s+(integer|string)\s+seeddatafile\s+(.*?)\s+"
-                            r"length\s+(\d+)\s+rows\s+(\d+)(\s+)?$",
-                            arg, re.IGNORECASE | re.DOTALL)
-        if matchObj:
-            m_DataType = str(matchObj.group(1)).lstrip().rstrip()
-            m_SeedFileName = str(matchObj.group(2)).lstrip().rstrip()
-            m_DataLength = int(matchObj.group(3))
-            m_nRows = int(matchObj.group(4))
-            Create_SeedCacheFile(p_szDataType=m_DataType, p_nDataLength=m_DataLength, p_nRows=m_nRows,
-                                 p_szSeedName=m_SeedFileName)
-            yield (
-                None,
-                None,
-                None,
-                None,
-                'seed file created Successful.')
+            for (title, result, headers, columntypes, status) in \
+                    self.DataHandler.Process_SQLCommand(arg, self.Result_Charset):
+                yield title, result, headers, columntypes, status
             return
 
         # 不认识的internal命令
@@ -1225,9 +1134,10 @@ class SQLCli(object):
                 # headers 包含原有语句的列名
                 # cur 是语句的执行结果
                 # output_format 输出格式
-                #   ascii              默认，即表格格式
+                #   ascii              默认，即表格格式(第三方工具实现，暂时保留以避免不兼容现象)
                 #   vertical           分行显示，每行、每列都分行
                 #   csv                csv格式显示
+                #   tab                表格形式（用format_output_tab自己编写)
                 formatted = self.format_output(
                     title, cur, headers, columntypes,
                     self.SQLOptions.get("OUTPUT_FORMAT").lower(),
@@ -1427,14 +1337,15 @@ class SQLCli(object):
             self.prompt_app = PromptSession()
 
         # 处理初始化启动文件，如果需要的话，在处理的过程中不打印任何日志信息
-        if self.profile is not None:
+        if len(self.profile) != 0:
             if "SQLCLI_DEBUG" not in os.environ:
                 self.SQLOptions.set("SILENT", "ON")
-            if "SQLCLI_DEBUG" in os.environ:
-                print("DEBUG:: Begin SQL profile [" + self.profile + "] ...")
-            self.DoSQL('start ' + self.profile)
-            if "SQLCLI_DEBUG" in os.environ:
-                print("DEBUG:: End SQL profile [" + self.profile + "]")
+            for m_Profile in self.profile:
+                if "SQLCLI_DEBUG" in os.environ:
+                    print("DEBUG:: Begin SQL profile [" + m_Profile + "] ...")
+                self.DoSQL('start ' + m_Profile)
+                if "SQLCLI_DEBUG" in os.environ:
+                    print("DEBUG:: End SQL profile [" + m_Profile + "]")
             if "SQLCLI_DEBUG" not in os.environ:
                 self.SQLOptions.set("SILENT", "OFF")
 
@@ -1476,6 +1387,10 @@ class SQLCli(object):
         # 退出进程
         self.echo("Disconnected.")
 
+        # 如果多进程管理器工作中，退出多进程管理器
+        if self.MultiProcessManager is not None:
+            self.MultiProcessManager.shutdown()
+
         # 还原进程标题
         setproctitle.setproctitle(m_Cli_ProcessTitleBak)
 
@@ -1502,8 +1417,9 @@ class SQLCli(object):
             # Unicode Error, This is console issue, Skip
             if "SQLCLI_DEBUG" in os.environ:
                 print("Console output error:: " + repr(ue))
-        if self.SpoolFileHandler is not None:
-            click.echo(s, file=self.SpoolFileHandler)
+        if len(self.SpoolFileHandler) != 0:
+            for m_SpoolFileHandler in self.SpoolFileHandler:
+                click.echo(s, file=m_SpoolFileHandler)
 
     def output(self, output, status=None):
         if output:
@@ -1580,6 +1496,81 @@ class SQLCli(object):
                     m_row = m_row + m_csv_delimiter
             yield str(m_row)
 
+    def format_output_tab(self, headers, columntypes, cur):
+        if self:
+            pass
+        # 将屏幕输出按照表格进行输出
+        # 记录每一列的最大显示长度
+        m_ColumnLength = []
+        # 首先将表头的字段长度记录其中
+        for m_Header in headers:
+            m_ColumnLength.append(len(m_Header))
+        # 查找列的最大字段长度
+        for m_Row in cur:
+            for m_nPos in range(0, len(m_Row)):
+                if isinstance(m_Row[m_nPos], str):
+                    for m_iter in m_Row[m_nPos].split('\n'):
+                        if len(m_iter) > m_ColumnLength[m_nPos]:
+                            m_ColumnLength[m_nPos] = len(m_iter)
+                else:
+                    if len(str(m_Row[m_nPos])) > m_ColumnLength[m_nPos]:
+                        m_ColumnLength[m_nPos] = len(str(m_Row[m_nPos]))
+        # 打印表格上边框
+        # 计算表格输出的长度, 开头有一个竖线，随后每个字段内容前有一个空格，后有一个空格加上竖线
+        # 1 + [（字段长度+3） *]
+        m_TableBoxLine = '+'
+        for m_Length in m_ColumnLength:
+            m_TableBoxLine = m_TableBoxLine + (m_Length + 2) * '-' + '+'
+        yield m_TableBoxLine
+        # 打印表头以及表头下面的分割线
+        m_TableContentLine = '|'
+        for m_nPos in range(0, len(headers)):
+            m_TableContentLine = m_TableContentLine + ' ' + \
+                                 str(headers[m_nPos]).center(m_ColumnLength[m_nPos]) + ' |'
+        yield m_TableContentLine
+        yield m_TableBoxLine
+        # 打印字段内容
+        for m_Row in cur:
+            # 首先计算改行应该打印的高度（行中的内容可能右换行符号）
+            m_RowHeight = 1
+            for m_nPos in range(0, len(m_Row)):
+                if isinstance(m_Row[m_nPos], str):
+                    if len(m_Row[m_nPos].split('\n')) > m_RowHeight:
+                        m_RowHeight = len(m_Row[m_nPos].split('\n'))
+            # 首先构造一个空的结果集，行数为计划打印的行高
+            m_output = []
+            if m_RowHeight == 1:
+                m_output.append(m_Row)
+            else:
+                for m_iter in range(0, m_RowHeight):
+                    m_output.append(())
+                # 依次填入数据
+                for m_nPos in range(0, len(m_Row)):
+                    m_SplitRow = ()
+                    if isinstance(m_Row[m_nPos], str):
+                        m_SplitColumnValue = m_Row[m_nPos].split('\n')
+                    else:
+                        m_SplitColumnValue = [m_Row[m_nPos], ]
+                    for m_iter in range(0, m_RowHeight):
+                        if len(m_SplitColumnValue) > m_iter:
+                            m_output[m_iter] = m_output[m_iter] + (m_SplitColumnValue[m_iter],)
+                        else:
+                            m_output[m_iter] = m_output[m_iter] + ("",)
+            for m_iter in m_output:
+                m_TableContentLine = '|'
+                for m_nPos in range(0, len(m_iter)):
+                    if columntypes[m_nPos] == str:
+                        # 字符串左对齐
+                        m_TableContentLine = m_TableContentLine + ' ' + \
+                                             str(m_iter[m_nPos]).ljust(m_ColumnLength[m_nPos]) + ' |'
+                    else:
+                        # 数值类型右对齐
+                        m_TableContentLine = m_TableContentLine + ' ' + \
+                                             str(m_iter[m_nPos]).rjust(m_ColumnLength[m_nPos]) + ' |'
+                yield m_TableContentLine
+        # 打印表格下边框
+        yield m_TableBoxLine
+
     def format_output(self, title, cur, headers, columntypes, p_format_name, max_width=None):
         output = []
 
@@ -1601,6 +1592,9 @@ class SQLCli(object):
             if p_format_name.upper() == 'CSV':
                 # 按照CSV格式输出查询结果
                 formatted = self.format_output_csv(headers, columntypes, cur)
+            if p_format_name.upper() == 'TAB':
+                # 按照TAB格式输出查询结果
+                formatted = self.format_output_tab(headers, columntypes, cur)
             else:
                 formatted = self.formatter.format_output(
                     cur,
@@ -1609,7 +1603,6 @@ class SQLCli(object):
                     column_types=None,
                     **output_kwargs
                 )
-
             if isinstance(formatted, str):
                 formatted = formatted.splitlines()
             formatted = iter(formatted)
@@ -1635,8 +1628,6 @@ class SQLCli(object):
 @click.option("--clientcharset", type=str, help="Set client charset. Default is UTF-8.")
 @click.option("--resultcharset", type=str, help="Set result charset. Default is same to clientcharset.")
 @click.option("--profile", type=str, help="Init profile.")
-@click.option("--reffile", type=str, help="Reference log file for compare result.")
-@click.option("--rptfile", type=str, help="Test report file.")
 def cli(
         version,
         logon,
@@ -1649,8 +1640,6 @@ def cli(
         clientcharset,
         resultcharset,
         profile,
-        reffile,
-        rptfile
 ):
     if version:
         print("Version:", __version__)
@@ -1678,9 +1667,7 @@ def cli(
         sqlperf=sqlperf,
         clientcharset=clientcharset,
         resultcharset=resultcharset,
-        profile=profile,
-        reffile=reffile,
-        rptfile=rptfile
+        profile=profile
     )
 
     # 运行主程序
